/*~FILE: AED_106.c
  (C) Copyright 2000 by SIGNALWARE Corporation.

  Property of: SIGNALWARE Corporation (sm)
               P.O. Box 50225
               Colorado Springs, CO 80949

  ~PRODUCT: AED-xxx - Analog Expansion Daughterboard
  ~VERSION: 1.0

  9/22/00 V1.0 - made from AED-103 Ver 0

  ~!

  ~HEADER:
     Application program for AED 103 demonstration employing expansion
   memory interface for access to the daughter board.
  ~!
  ~WARNING:
    This code should be modified only as indicated to implement
    various applications. When this code is changed to implement
    an application, please change the name of the module to the
    name of the project to avoid confusion.  The name for
    PROJECT in the make file will also have to be changed.
   ~! */

#ifdef PC
#define cregister /* nil */
#define interrupt /* nil */
#define noprint  0
#else
#define noprint  0 /* set to 1 to remove all printing */
#endif

/* #include <stdlib.h> */
#include <stdio.h>
#include <string.h>
#include <common.h>
#include <board.h>
#include <dma.h>
#include <intr.h>
#include <emif.h>
#include "dma_ex.h"
#include "appl_hdr.h"

/* WITH_DB = 1 is normal operation;
   WITH_DB undefined is test w/o daughterboard */
#define WITH_DB 1

#define FPGA_IO_REG (unsigned int *)(0x01500000)

/* application may change these defines */
#define NO_RECORDS  128    /* records/block */
#define NO_BLOCKS   3      /* blocks/buffer */
#define RECLEN 8           /* length of record */
#define DIVIDE_POWER (7)   /* NO_RECORDS = 2^7 */
#define ELEMENTSIZE 4      /* bytes in 32 bit word */
#define RECSIZE (RECLEN*ELEMENTSIZE)
#define ELEMENTSIZE_CODE DMA_ESIZE32 
    /* two half words in each transfer */
#define SAVE_RECORDS 16   /* size of printout */
#define ITERATIONS 10      /* size of averages */

#ifndef WITH_DB
ApplBlockType test_data;
#endif
ApplBlockType save_data;

static unsigned int fpga_io_reg = 0x70000001;

/*---------------------------------------------*/
/* the following code is application dependent */
#ifndef WITH_DB
static char test_data_alloc[NO_BLOCKS*NO_RECORDS*RECSIZE];
#endif
static char save_data_alloc[SAVE_RECORDS*RECSIZE];

static int loop_count;
static int iteration;
static unsigned int A_value[ITERATIONS];
static unsigned int B_value[ITERATIONS];
static unsigned int C_value[ITERATIONS];
static unsigned int D_value[ITERATIONS];
static unsigned int E_value[ITERATIONS];
static unsigned int F_value[ITERATIONS];
static unsigned int G_value[ITERATIONS];
static unsigned int H_value[ITERATIONS];
static unsigned int I_value[ITERATIONS];
static unsigned int J_value[ITERATIONS];
static unsigned int K_value[ITERATIONS];
static unsigned int L_value[ITERATIONS];
static unsigned int M_value[ITERATIONS];
static unsigned int N_value[ITERATIONS];
static unsigned int O_value[ITERATIONS];
static unsigned int P_value[ITERATIONS];

static ApplBlockType addr[20] =  /* for testing */
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static int bufs[20] =  /* for testing */
   {99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
    99, 99, 99, 99, 99, 99, 99, 99, 99, 99};
/* end of application dependent code */
/*---------------------------------------------*/

static void blink_leds(int n);
static void sample_buffer(ApplBlockType buffer, int n, int m);
#ifndef WITH_DB
static void build_test_buffer(ApplBlockType buffer, int n, int m);
#endif


/****************************************************************/
/* Return constants for the application parameters.             */
/****************************************************************/
void appl_parms(unsigned int *blocks, unsigned int *records,
                unsigned int *reclen, unsigned int *size,
                unsigned int *mode) {
  /*---------------------------------------------*/
  /* the following code may be application dependent or
     additional code may be required */
 
   
  EMIF_CE1_CTRL = 0x40FD0220;

  *blocks = NO_BLOCKS;
  *records = NO_RECORDS;
  *reclen = RECLEN;
  *size = ELEMENTSIZE_CODE; 
  *mode = DMA_FS_MODE;
  /* end of application dependent code */
  /*---------------------------------------------*/


    
} /* end appl_parms */
/****************************************************************/
/* Clear buffers, build test data and set DMA to send test data.*/
/****************************************************************/



void appl_init(ApplBlockType data_buffer) {
  /*---------------------------------------------*/
  /* the following code may be application dependent or additional
     code may be required */
  int i, j, k;
  unsigned int *ptr;
  unsigned int pattern;

#ifndef WITH_DB
  test_data.byte = test_data_alloc;
#endif
  save_data.byte = save_data_alloc;

  ptr = data_buffer.uword;
  for (i=0; i<NO_BLOCKS; i++) for (j=0; j<NO_RECORDS; j++) 
    for (k=0; k<RECLEN; k++) *(ptr++) = 0;
  
  ptr = save_data.uword;
  for (j=0; j<SAVE_RECORDS; j++) for (k=0; k<RECLEN; k++) 
    *(ptr++) = 0;

  for (j=0; j<ITERATIONS; j++) {
    A_value[j] = 0;
    B_value[j] = 0;
    C_value[j] = 0;
    D_value[j] = 0;
    E_value[j] = 0;
    F_value[j] = 0;
    G_value[j] = 0;
    H_value[j] = 0;
    I_value[j] = 0;
    J_value[j] = 0;
    K_value[j] = 0;
    L_value[j] = 0;
    M_value[j] = 0;
    N_value[j] = 0;
    O_value[j] = 0;
    P_value[j] = 0;

  } /* end for */

  loop_count = 0;
  iteration = 0;
  
  /* Test the digital I/O by turning on each output
     one at a time for one second. */
  pattern = 1;
  fpga_io_reg = 0x70000001;
  *FPGA_IO_REG = fpga_io_reg;
  blink_leds(1);
  for (j=0; j<24; j++) {
    fpga_io_reg &= ~pattern;
    pattern <<= 1;
    fpga_io_reg |= pattern;
    *FPGA_IO_REG = fpga_io_reg;
    blink_leds(1);
  } /* end for */
  
  /* Additional test patterns for the digital I/O */
/*for (j=0; j<16; j++) {
    fpga_io_reg = 0x70AAAAAA;
    *FPGA_IO_REG = fpga_io_reg;
    delay_msec(100);
    fpga_io_reg = 0x70555555;
    *FPGA_IO_REG = fpga_io_reg;
    delay_msec(100);
  }  
    
  for (i=0; i<5; i++) {
    pattern = 1;
    fpga_io_reg = 0x70000000;  
    for (j=0; j<24; j++) {
      fpga_io_reg |= pattern;
      *FPGA_IO_REG = fpga_io_reg;
      pattern = (pattern << 1) + 1;
      delay_msec(20);      
    } 
  
    for (j=0; j<24; j++) {
      pattern = (pattern - (1 << j));
      fpga_io_reg &= (pattern | 0x70000000);
      *FPGA_IO_REG = fpga_io_reg;
      delay_msec(20);      
    } 
  }   
  
  for (i=0; i<8; i++) {
    pattern = 0x0;
    for (j=0; j<8; j++) {
      pattern = (((pattern & 0xFF0000) + (0x10000 << j)) | ((pattern & 0xFF00) + (0x100 << j)) | ((pattern & 0xFF) + (1 << j)));
      fpga_io_reg |= pattern;
      *FPGA_IO_REG = fpga_io_reg;
      delay_msec(20);
    } 
  
    for (j=0; j<8; j++) {
      pattern = (((pattern & 0xFF0000) - (0x10000 << j)) | ((pattern & 0xFF00) - (0x100 << j)) | ((pattern & 0xFF) - (1 << j)));
      fpga_io_reg &= (pattern | 0x70000000);
      *FPGA_IO_REG = fpga_io_reg;
      delay_msec(20);
    }  
    
    for (j=0; j<8; j++) {
      pattern = (((pattern & 0xFF0000) + (0x800000 >> j)) | ((pattern & 0xFF00) + (0x8000 >> j)) | ((pattern & 0xFF) + (0x80 >> j)));
      fpga_io_reg |= pattern;
      *FPGA_IO_REG = fpga_io_reg;
      delay_msec(20);
    }
  
    for (j=0; j<8; j++) {
      pattern = (((pattern & 0xFF0000) - (0x800000 >> j)) | ((pattern & 0xFF00) - (0x8000 >> j)) | ((pattern & 0xFF) - (0x80 >> j)));
      fpga_io_reg &= (pattern | 0x70000000);
      *FPGA_IO_REG = fpga_io_reg;
      delay_msec(20);
    }   
  }*/ 
  
  /* end of application dependent code */
  /*---------------------------------------------*/
  /* the following code line is removed when daughterboard is
     added */
#ifndef WITH_DB
  build_test_buffer(test_data, NO_RECORDS*RECLEN, NO_BLOCKS);
#endif
} /* end appl_init */
/****************************************************************/
/* Process a buffer of data signaled ready from main            */
/****************************************************************/
int appl_process(ApplBlockType data_block, int buf_number) {
  int ret;
  /*---------------------------------------------*/
  /* the following code is application dependent */
  int j;
  unsigned int sumA = 0, sumB = 0, sumC = 0, sumD = 0;
  unsigned int sumE = 0, sumF = 0, sumG = 0, sumH = 0;
  unsigned int sumI = 0, sumJ = 0, sumK = 0, sumL = 0;
  unsigned int sumM = 0, sumN = 0, sumO = 0, sumP = 0;
  ApplBlockType ptr;

  if (iteration==0) {
    sample_buffer(data_block, SAVE_RECORDS, RECLEN);
  } /* end if */
  addr[iteration] = data_block;
  bufs[iteration] = buf_number;

  /* compute the buffer mean */
  ptr = data_block;
  for (j=0; j<NO_RECORDS/16; j++) {
    sumA += (unsigned int)*(ptr.uhword++);
    sumB += (unsigned int)*(ptr.uhword++);
    sumC += (unsigned int)*(ptr.uhword++);
    sumD += (unsigned int)*(ptr.uhword++);
    sumE += (unsigned int)*(ptr.uhword++);
    sumF += (unsigned int)*(ptr.uhword++);
    sumG += (unsigned int)*(ptr.uhword++);
    sumH += (unsigned int)*(ptr.uhword++);
    sumI += (unsigned int)*(ptr.uhword++);
    sumJ += (unsigned int)*(ptr.uhword++);
    sumK += (unsigned int)*(ptr.uhword++);
    sumL += (unsigned int)*(ptr.uhword++);
    sumM += (unsigned int)*(ptr.uhword++);
    sumN += (unsigned int)*(ptr.uhword++);
    sumO += (unsigned int)*(ptr.uhword++);
    sumP += (unsigned int)*(ptr.uhword++);
  } /* end for */

  A_value[iteration] = sumA>>(DIVIDE_POWER-4);
  B_value[iteration] = sumB>>(DIVIDE_POWER-4);
  C_value[iteration] = sumC>>(DIVIDE_POWER-4);
  D_value[iteration] = sumD>>(DIVIDE_POWER-4);
  E_value[iteration] = sumE>>(DIVIDE_POWER-4);
  F_value[iteration] = sumF>>(DIVIDE_POWER-4);
  G_value[iteration] = sumG>>(DIVIDE_POWER-4);
  H_value[iteration] = sumH>>(DIVIDE_POWER-4);
  I_value[iteration] = sumI>>(DIVIDE_POWER-4);
  J_value[iteration] = sumJ>>(DIVIDE_POWER-4);
  K_value[iteration] = sumK>>(DIVIDE_POWER-4);
  L_value[iteration] = sumL>>(DIVIDE_POWER-4);
  M_value[iteration] = sumM>>(DIVIDE_POWER-4);
  N_value[iteration] = sumN>>(DIVIDE_POWER-4);
  O_value[iteration] = sumO>>(DIVIDE_POWER-4);
  P_value[iteration] = sumP>>(DIVIDE_POWER-4);

  ret = ++iteration > (ITERATIONS-1)? 99:0;
  /* end of application dependent code */
  /*---------------------------------------------*/
  return (ret);
} /* end appl_process */
/****************************************************************/
/* Fills idle time by incrementing loop count                   */
/****************************************************************/
int appl_idle(void) {
  /*---------------------------------------------*/
  /* the following code is application dependent */
  loop_count++;
  /* end of application dependent code */
  /*---------------------------------------------*/
  return 0;
} /* end appl_idle */
/****************************************************************/
/* Prints the values in the first 128 words of test_data        */
/****************************************************************/
void appl_end(void) {
  /*---------------------------------------------*/
  /* the following code is application dependent */
  int j;
  unsigned int pattern;

  printf("Block Addresses %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d)\n"
         "                %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d)\n"
         "                %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d)\n"
         "                %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d) %08x(%2d)\n",
    addr[0].byte, bufs[0], addr[1].byte, bufs[1],
    addr[2].byte, bufs[2], addr[3].byte, bufs[3], addr[4].byte, bufs[4],
    addr[5].byte, bufs[5], addr[6].byte, bufs[6],
    addr[7].byte, bufs[7], addr[8].byte, bufs[8], addr[9].byte, bufs[9],
    addr[10].byte, bufs[10], addr[11].byte, bufs[11],
    addr[12].byte, bufs[12], addr[13].byte, bufs[13], addr[14].byte, bufs[14],
    addr[15].byte, bufs[15], addr[16].byte, bufs[16],
    addr[17].byte, bufs[17], addr[18].byte, bufs[18],
    addr[19].byte, bufs[19]);  /* for testing */

  printf ("A= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    A_value[0], A_value[1], A_value[2], A_value[3], A_value[4],
    A_value[5], A_value[6], A_value[7], A_value[8], A_value[9]);
  printf ("B= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    B_value[0], B_value[1], B_value[2], B_value[3], B_value[4],
    B_value[5], B_value[6], B_value[7], B_value[8], B_value[9]);
  printf ("C= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    C_value[0], C_value[1], C_value[2], C_value[3], C_value[4],
    C_value[5], C_value[6], C_value[7], C_value[8], C_value[9]);
  printf ("D= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    D_value[0], D_value[1], D_value[2], D_value[3], D_value[4],
    D_value[5], D_value[6], D_value[7], D_value[8], D_value[9]);
  printf ("E= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    E_value[0], E_value[1], E_value[2], E_value[3], E_value[4],
    E_value[5], E_value[6], E_value[7], E_value[8], E_value[9]);
  printf ("F= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    F_value[0], F_value[1], F_value[2], F_value[3], F_value[4],
    F_value[5], F_value[6], F_value[7], F_value[8], F_value[9]);
  printf ("G= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    G_value[0], G_value[1], G_value[2], G_value[3], G_value[4],
    G_value[5], G_value[6], G_value[7], G_value[8], G_value[9]);
  printf ("H= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    H_value[0], H_value[1], H_value[2], H_value[3], H_value[4],
    H_value[5], H_value[6], H_value[7], H_value[8], H_value[9]);
  printf ("I= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    I_value[0], I_value[1], I_value[2], I_value[3], I_value[4],
    I_value[5], I_value[6], I_value[7], I_value[8], I_value[9]);
  printf ("J= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    J_value[0], J_value[1], J_value[2], J_value[3], J_value[4],
    J_value[5], J_value[6], J_value[7], J_value[8], J_value[9]);
  printf ("K= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    K_value[0], K_value[1], K_value[2], K_value[3], K_value[4],
    K_value[5], K_value[6], K_value[7], K_value[8], K_value[9]);
  printf ("L= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    L_value[0], L_value[1], L_value[2], L_value[3], L_value[4],
    L_value[5], L_value[6], L_value[7], L_value[8], L_value[9]);
  printf ("M= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    M_value[0], M_value[1], M_value[2], M_value[3], M_value[4],
    M_value[5], M_value[6], M_value[7], M_value[8], M_value[9]);
  printf ("N= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    N_value[0], N_value[1], N_value[2], N_value[3], N_value[4],
    N_value[5], N_value[6], N_value[7], N_value[8], N_value[9]);
  printf ("O= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    O_value[0], O_value[1], O_value[2], O_value[3], O_value[4],
    O_value[5], O_value[6], O_value[7], O_value[8], O_value[9]);
  printf ("P= %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
    P_value[0], P_value[1], P_value[2], P_value[3], P_value[4],
    P_value[5], P_value[6], P_value[7], P_value[8], P_value[9]);

  for (j=0; j<SAVE_RECORDS*2; j+=2) {
    printf("[%3d] %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x\n",
          j*RECLEN,
             save_data.uhword[j*RECLEN],
             save_data.uhword[j*RECLEN+1],
             save_data.uhword[j*RECLEN+2],
             save_data.uhword[j*RECLEN+3],
             save_data.uhword[j*RECLEN+4],
             save_data.uhword[j*RECLEN+5],
             save_data.uhword[j*RECLEN+6],
             save_data.uhword[j*RECLEN+7],
             save_data.uhword[j*RECLEN+8],
             save_data.uhword[j*RECLEN+9],
             save_data.uhword[j*RECLEN+10],
             save_data.uhword[j*RECLEN+11],
             save_data.uhword[j*RECLEN+12],
             save_data.uhword[j*RECLEN+13],
             save_data.uhword[j*RECLEN+14],
             save_data.uhword[j*RECLEN+15]);
  } /* end for */

  blink_leds(3);
  pattern = 1;
  for (j=0; j<27; j++) {
    fpga_io_reg &= ~pattern;
    pattern <<= 1;
    fpga_io_reg |= pattern;
    *FPGA_IO_REG = fpga_io_reg;
    blink_leds(1);
  } /* end for */

  /* end of application dependent code */
  /*---------------------------------------------*/
} /* end appl_end */
/*==============================================================*/
/*  Save a sample buffer for printout at completions            */
/*==============================================================*/
static void sample_buffer(ApplBlockType first, int n, int m) {
  int j;

  for (j=0; j<n*m; j++)  {
       save_data.uword[j] =
              first.uword[j];
  } /* end for */
} /* end build_test_buffer */
/****************************************************************/
/* Blinking light emitting diodes                               */
/****************************************************************/
static void blink_leds(int n) {
int i;
  for (i=0;i<n;i++)
  {
    evm_led_enable(0);
    evm_led_enable(1);
    delay_msec(500);
    evm_led_disable(0);
    evm_led_disable(1);
    delay_msec(500);
  }
} /* end blink leds*/
/*==============================================================*/
/* This code is only for testing; it 1s removed once the daughter
   board is added and the call in appl_init is removed. */
#ifndef WITH_DB
static void build_test_buffer(ApplBlockType fill, int n, int m) {
  /*---------------------------------------------*/
  /* the following code is application dependent */
  int i, j;

  /* fill can be changed to accomodate testing */
  for (i=0; i<m; i++) for (j=0; j<n; j++)  {
       fill.uword[i*n+j] = (0x00010003*(j%4+1));
  } /* end for */
  /* end of application dependent code */
  /*---------------------------------------------*/

  /* set up dma channel to read from fill in lieu of FPGA */
  test_DMA_channel (fill.byte);
} /* end build_test_buffer */
#endif

